package ru.dbotthepony.mc.otm.block.entity.storage

import net.minecraft.MethodsReturnNonnullByDefault
import net.minecraft.core.BlockPos
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.world.Container
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.ServerConfig
import ru.dbotthepony.mc.otm.block.IDroppableContainer
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.block.entity.MatteryPoweredBlockEntity
import ru.dbotthepony.mc.otm.block.storage.DriveViewerBlock
import ru.dbotthepony.mc.otm.block.entity.WorkerState
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.core.map
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.registry.MBlocks
import javax.annotation.ParametersAreNonnullByDefault

class DriveViewerBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) : MatteryPoweredBlockEntity(MBlockEntities.DRIVE_VIEWER, p_155229_, p_155230_), IDroppableContainer {
	override fun setChanged() {
		super.setChanged()

		tickOnceServer {
			var state = blockState

			if (container.getItem(0).getCapability(MatteryCapability.DRIVE).isPresent && energy.batteryLevel >= OverdriveThatMatters.INSTANCE.ITEM_STORAGE().energyPerOperation) {
				state = state.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING)
			} else {
				state = state.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE)
			}

			if (state !== blockState) {
				it.setBlock(blockPos, state, Block.UPDATE_CLIENTS)
			}
		}
	}

	override val energy = WorkerEnergyStorage(this, ServerConfig.DRIVE_VIEWER)

	val container: MatteryContainer = object : MatteryContainer(this::setChanged, 1) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)

			tickOnceServer {
				if (!isRemoved) {
					var state = blockState

					if (new.getCapability(MatteryCapability.DRIVE).isPresent) {
						state = state.setValue(DriveViewerBlock.DRIVE_PRESENT, true)
					} else {
						state = state.setValue(DriveViewerBlock.DRIVE_PRESENT, false)
					}

					if (state !== blockState) {
						it.setBlock(blockPos, state, Block.UPDATE_CLIENTS)
					}
				}
			}
		}
	}

	override val droppableContainer: Container
		get() = container
	override val defaultDisplayName: Component
		get() = MBlocks.DRIVE_VIEWER.name

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu? {
		return DriveViewerMenu(containerID, inventory, this)
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt[INVENTORY_KEY] = container.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		nbt.map(INVENTORY_KEY, container::deserializeNBT)
	}

	fun tick() {
		batteryChargeLoop()
	}
}