package ru.dbotthepony.mc.otm.core

import java.util.function.Consumer
import java.util.function.Supplier
import kotlin.reflect.KMutableProperty0
import kotlin.reflect.KProperty

inline var <V> GetterSetter<V>.value: V
	get() = get()
	set(value) { accept(value) }

@Suppress("nothing_to_inline")
inline operator fun <V> GetterSetter<V>.getValue(thisRef: Any, property: KProperty<*>): V {
	return get()
}

@Suppress("nothing_to_inline")
inline operator fun <V> GetterSetter<V>.setValue(thisRef: Any, property: KProperty<*>, value: V) {
	accept(value)
}

interface GetterSetter<V> : Supplier<V>, Consumer<V> {
	companion object {
		fun <V> of(getter: Supplier<V>, setter: Consumer<V>): GetterSetter<V> {
			return object : GetterSetter<V> {
				override fun get(): V {
					return getter.get()
				}

				override fun accept(t: V) {
					setter.accept(t)
				}
			}
		}

		fun <V> of(getter: () -> V, setter: (V) -> Unit): GetterSetter<V> {
			return object : GetterSetter<V> {
				override fun get(): V {
					return getter.invoke()
				}

				override fun accept(t: V) {
					setter.invoke(t)
				}
			}
		}

		fun <V> of(property: KMutableProperty0<V>): GetterSetter<V> {
			return object : GetterSetter<V> {
				override fun get(): V {
					return property.get()
				}

				override fun accept(t: V) {
					property.set(t)
				}
			}
		}
	}
}

fun <V> KMutableProperty0<V>.asGetterSetter() = GetterSetter.of(this)
fun <V> KMutableProperty0<V>.asGetterOnly() = GetterSetter.of(Supplier { this.get() }, Consumer { /* do nothing */ })
